# generic entity in the platform thingie for the HoV
# all other entities inherit this

from std import *

from constants import *
import system

class struct:
    pass

class Ent:
    def __init__(self,x=0,y=0):
        global state

        self.x=x
        self.y=y
        self.vx=0.0
        self.vy=0.0
        self.state=state.STAND
        self.dir=LEFT
        self.killflag=0
        self.count=0
        self.isobs=0                    # if true, entities can walk upon this entity

        self.leftanim=[]
        self.rightanim=[]

    def __del__(self):
        del map.entities[`id(self)`]

#-----------------------------------------------------------

    def CreateCHR(self,chrname):
        self.e=Entity(self.x,self.y,chrname)
        self.width=self.e.hotwidth
        self.height=self.e.hotheight
        map.entities[`id(self)`]=self.e

#-----------------------------------------------------------

    def SetState(self,newstate):
        if self.state==newstate:
            return

        if self.state>=state.IMMOBILE and self.count>0:
            return

        self.state=newstate
        if self.dir==LEFT:
            anim=self.leftanim
        else:
            anim=self.rightanim
        if newstate<len(anim):
            self.e.Move(anim[newstate])
#            Log(anim[newstate])
        else:
            self.e.Move('Z0')
#            Log('Z1')

#-----------------------------------------------------------

    def Stop(self):
        #Log('stop')
        self.vx=0
        if self.state==state.WALK:
            self.SetState(state.STAND)

#-----------------------------------------------------------

    def Update(self):
        if self.state==state.JUMP or self.state==state.FALL:
            friction=AIRFRICTION
        else:
            friction=GROUNDFRICTION

        if self.vx>0:
            if self.vx>friction:
                self.vx-=friction
            else:
                self.Stop()
        elif self.vx<0:
            if self.vx<-friction:
                self.vx+=friction
            else:
                self.Stop()

        self.ProcessAI()
        self.HandleGravity()
        self.HandleVerticalVelocity()
        self.HandleHorizontalVelocity()

        self.x+=self.vx
        self.y+=self.vy
        self.e.x=self.x
        self.e.y=self.y

#-----------------------------------------------------------

    def ProcessAI(self):
        if self.e.y+self.height>=map.height*map.tileheight:
            self.killflag=1

#-----------------------------------------------------------

    def HandleGravity(self):
        self.vy+=GRAVITY

#-----------------------------------------------------------
    def HandleVerticalVelocity(self):
        if not self.vy:
            return

        x1=self.x
        x2=self.x+self.width

        if self.vy>0:
            y=self.y+self.height
        elif self.vy<0:
            y=self.y-1

        for x in range(x1,x2+1,map.tilewidth):
            if map.GetObs(x/map.tilewidth,y/map.tileheight):
                if self.vy>0:
                    if self.state!=state.WALK:
                        self.SetState(state.STAND)
                    self.y-=self.y%map.tileheight
                elif self.vy<0:
                    self.y+=map.tileheight-self.y%map.tileheight

                self.vy=0
                return
            e=system.EntObs(x,y)
            if e!=None:
                if self.vy>0:
                    if self.state==state.FALL:
                        self.SetState(state.STAND)
                    self.y=e.y-self.height+1
                elif self.vy<0:
                    self.y=e.y+e.height

                self.vy=0
                return

        if self.vy>0:
            self.SetState(state.FALL)
        else:
            self.SetState(state.JUMP)
#- <Ear> ---------------------------------------------------

    def TestHorizontal(self):
        x=self.x+self.vx
        y1=self.y
        y2=self.y+self.height-1
        if self.vx>0:
            x+=self.width

        for y in range(y1,y2+1):
            if map.GetObs(x/map.tilewidth,y/map.tileheight):
                return 1
            e=system.EntObs(x,y)
            if e!=None:
                return 1

        return 0

#-----------------------------------------------------------

    def HandleHorizontalVelocity(self):
        # horizontal momentum
        if self.vx==0:
            return

        if self.TestHorizontal():
            self.Stop()
            return

        if self.state==state.STAND:
            self.SetState(state.WALK)

#-----------------------------------------------------------

    def CheckCollision(self,e):
        if self==e:
            return
        if self.x>e.x+e.width:
            return
        if self.x+self.width<e.x:
            return
        if self.y>e.y+e.height:
            return
        if self.y+self.height<e.y:
            return

        self.HandleCollision(e)

#-----------------------------------------------------------

    def HandleCollision(self,e):                # e is the other involved in the collision
        if self==e:
            Log('Something wrong with the collision stuff')
            return                              # -_-
        pass

#-----------------------------------------------------------

    def HealHP(self,amount):                    # stub
        pass

#-----------------------------------------------------------

    def Hurt(self,amount):                    # stub
        pass

#- <Ear> ---------------------------------------------------

    def MoveLeft(self):
        self.dir=LEFT
        if self.state==state.FALL or self.state==state.JUMP:
            self.vx-=AIRSPEED
        elif self.state<state.IMMOBILE:
            self.vx-=GROUNDSPEED

#- <Ear> ---------------------------------------------------

    def MoveRight(self):
        self.dir=RIGHT
        if self.state==state.FALL or self.state==state.JUMP:
            self.vx+=AIRSPEED
        elif self.state<state.IMMOBILE:
            self.vx+=GROUNDSPEED