# player class

from std import *
from constants import *
from entity import Ent
from gameover import GameOver

class Player(Ent):
    def __init__(self,x=0,y=0):
        Ent.__init__(self,x,y)          # parent constructor

        self.JUMPVELOCITY=2.4
        self.SCYTHERANGE=32
        self.MAXHP=50
        self.MAXEP=25

        self.HP=50
        self.EP=25

        self.LIVES=3

        self.curweapon=weap.SCYTHE

        self.scytheimageleft=Image()
        self.scytheimageleft.Load('scytheleft.png')
        self.scytheimageright=Image()
        self.scytheimageright.Load('scytheright.png')

        self.CreateCHR('deflator.chr')

        self.leftanim=[
            # standing
            'Z4',
            # walking
            'Z5W20 Z6W20 Z7W20 Z6W20 B',
            # jumping
            'Z9',
            # falling
            'Z9',
            # attacking
            'Z4',
            # hurt
            'Z9'
            ]

        self.rightanim=[
            # standing
            'Z0',
            # walking
            'Z1W20 Z2W20 Z3W20 Z2W20 B',
            # jumping
            'Z8',
            # falling
            'Z8',
            # Attacking
            'Z0',
            # hurt
            'Z8'
            ]

#-----------------------------------------------------------

    def ProcessAI(self):
        if self.state==state.ATTACK:
            self.HandleScythe()

        if self.state==state.HURT or self.state==state.ATTACK:
            self.count-=1
            if self.count>0:
                return
            self.SetState(state.FALL)

        if input.left:
            self.MoveLeft()
        if input.right:
            self.MoveRight()

        if input.Button(JUMPBUTTON) and (self.state==state.WALK or self.state==state.STAND):
            self.vy=-self.JUMPVELOCITY
            self.y-=1
            self.SetState(state.JUMP)

        if not input.Button(JUMPBUTTON) and self.vy<-0.1:
            self.vy+=0.1

        if self.state==state.FALL:
            input.SetButton(JUMPBUTTON,0)

        if input.Button(ATTACKBUTTON) and self.state<state.IMMOBILE:
            self.SetState(state.ATTACK)
            self.count=50

        if self.vx>MAXPLAYERSPEED:
            self.vx=MAXPLAYERSPEED
        elif self.vx<-MAXPLAYERSPEED:
            self.vx=-MAXPLAYERSPEED

        if self.y>map.height*map.tileheight:
            self.Death()

#-----------------------------------------------------------

    def HandleScythe(self):
        from system import ents

        x1=self.x
        y1=self.y
        if self.dir==LEFT:
            x1-=self.width

        x2=x1+SCYTHERANGE
        y2=y1+self.height

        for e in ents:
            if not isinstance(e,Player):
                if e.x+e.width>x1 and e.y+e.height>y1 and e.x<x2 and e.y<y2:
                    e.Hurt(SCYTHEPOWER)

#-----------------------------------------------------------

    def HealHP(self,amount):
        self.HP+=amount
        if self.HP>self.MAXHP:
            self.HP=self.MAXHP

#-----------------------------------------------------------

    def Hurt(self,amount):
        if self.state==state.HURT:       # still in recoil from the last hit
            return

        if amount>=self.HP:
            self.HP=0
            # show death animation
            self.Death()

        self.HP-=amount
        self.SetState(state.HURT)
        self.count=100
        if self.dir==LEFT:
            self.vx=2
        else:
            self.vx=-2

#- <Ear> ---------------------------------------------------

    def GainLife(self):
        self.LIVES+=1

#- <Ear> ---------------------------------------------------

    def Death(self):
        self.LIVES-=1
        if self.LIVES<0:
            GameOver()
        self.HP=self.MAXHP
        self.EP=self.MAXEP

#- <Ear> ---------------------------------------------------

    def RestoreEP(self,amount):
        self.EP+=amount
        if self.EP>self.MAXEP:
            self.EP=self.MAXEP

#- <Ear> ---------------------------------------------------

    def UseEP(self,amount):
        if not self.EP:
            return 0
        self.EP-=amount
        if self.EP<0:
            self.EP=0

#- <Ear> ---------------------------------------------------

    def IncreaseHP(self,amount):
        self.MAXHP+=amount
        self.HP+=amount

#- <Ear> ---------------------------------------------------

    def IncreaseEP(self,amount):
        self.MAXEP+=amount
        self.EP+=amount
